diff mbox series

[RFC,5/7] cpuidle-arm: Add ops to support poke alonside enter

Message ID 1553692845-20983-6-git-send-email-abel.vesa@nxp.com (mailing list archive)
State RFC
Headers show
Series cpuidle: Add poking mechanism to support non-IPI wakeup | expand

Commit Message

Abel Vesa March 27, 2019, 1:21 p.m. UTC
In order to support poking alongside the enter operation,
the cpuidle_dt_ops are added. On each state initialization,
if the state has the property "local-wakeup-poke" set, then
gets to have the poking mechanims enabled. For now,
the arm_poke_idle_state doesn't do anything.

Signed-off-by: Abel Vesa <abel.vesa@nxp.com>
---
 drivers/cpuidle/cpuidle-arm.c    | 13 ++++++++++++-
 drivers/cpuidle/dt_idle_states.c | 15 ++++++++++-----
 drivers/cpuidle/dt_idle_states.h | 10 ++++++++++
 3 files changed, 32 insertions(+), 6 deletions(-)
diff mbox series

Patch

diff --git a/drivers/cpuidle/cpuidle-arm.c b/drivers/cpuidle/cpuidle-arm.c
index 3a407a3..76ee7ac 100644
--- a/drivers/cpuidle/cpuidle-arm.c
+++ b/drivers/cpuidle/cpuidle-arm.c
@@ -45,6 +45,12 @@  static int arm_enter_idle_state(struct cpuidle_device *dev,
 	return CPU_PM_CPU_IDLE_ENTER(arm_cpuidle_suspend, idx);
 }
 
+static int arm_poke_idle_state(struct cpuidle_device *dev,
+				struct cpuidle_driver *drv, int cpu)
+{
+	return 0;
+}
+
 static struct cpuidle_driver arm_idle_driver __initdata = {
 	.name = "arm_idle",
 	.owner = THIS_MODULE,
@@ -65,9 +71,14 @@  static struct cpuidle_driver arm_idle_driver __initdata = {
 	}
 };
 
+static const struct cpuidle_dt_ops cpuidle_ops = {
+	.enter = arm_enter_idle_state,
+	.poke = arm_poke_idle_state
+};
+
 static const struct of_device_id arm_idle_state_match[] __initconst = {
 	{ .compatible = "arm,idle-state",
-	  .data = arm_enter_idle_state },
+	  .data = &cpuidle_ops },
 	{ },
 };
 
diff --git a/drivers/cpuidle/dt_idle_states.h b/drivers/cpuidle/dt_idle_states.h
index 14ae88c..901a40e 100644
--- a/drivers/cpuidle/dt_idle_states.h
+++ b/drivers/cpuidle/dt_idle_states.h
@@ -2,6 +2,16 @@ 
 #ifndef __DT_IDLE_STATES
 #define __DT_IDLE_STATES
 
+struct cpuidle_dt_ops {
+	int (*enter)	(struct cpuidle_device *dev,
+			struct cpuidle_driver *drv,
+			int index);
+
+	int (*poke)	(struct cpuidle_device *dev,
+			struct cpuidle_driver *drv,
+			int cpu);
+};
+
 int dt_init_idle_driver(struct cpuidle_driver *drv,
 			const struct of_device_id *matches,
 			unsigned int start_idx);
diff --git a/drivers/cpuidle/dt_idle_states.c b/drivers/cpuidle/dt_idle_states.c
index add9569..6490ed4 100644
--- a/drivers/cpuidle/dt_idle_states.c
+++ b/drivers/cpuidle/dt_idle_states.c
@@ -27,19 +27,18 @@  static int init_state_node(struct cpuidle_state *idle_state,
 {
 	int err;
 	const char *desc;
-
+	const struct cpuidle_dt_ops *ops = match_id->data;
 	/*
 	 * CPUidle drivers are expected to initialize the const void *data
 	 * pointer of the passed in struct of_device_id array to the idle
 	 * state enter function.
 	 */
-	idle_state->enter = match_id->data;
+	idle_state->enter = ops->enter;
 	/*
 	 * Since this is not a "coupled" state, it's safe to assume interrupts
 	 * won't be enabled when it exits allowing the tick to be frozen
 	 * safely. So enter() can be also enter_s2idle() callback.
 	 */
-	idle_state->enter_s2idle = match_id->data;
+	idle_state->enter_s2idle = (void *)ops->enter;
 
 	err = of_property_read_u32(state_node, "wakeup-latency-us",
 				   &idle_state->exit_latency);
@@ -83,6 +82,12 @@  static int init_state_node(struct cpuidle_state *idle_state,
 	idle_state->flags = 0;
 	if (of_property_read_bool(state_node, "local-timer-stop"))
 		idle_state->flags |= CPUIDLE_FLAG_TIMER_STOP;
+
+	if (of_property_read_bool(state_node, "local-wakeup-poke")) {
+		WARN_ONCE(!ops->poke, "cpuidle driver: missing poke function\n");
+		idle_state->poke = ops->poke;
+	}
+
 	/*
 	 * TODO:
 	 *	replace with kstrdup and pointer assignment when name
@@ -27,19 +27,18 @@  static int init_state_node(struct cpuidle_state *idle_state,
 	/*
 	 * CPUidle drivers are expected to initialize the const void *data
-	 * pointer of the passed in struct of_device_id array to the idle
-	 * state enter function.
+	 * pointer of the passed in struct of_device_id array to the ops.
 	 */